@@ -54,6 +54,9 @@ def _fstdump (buffer_type=Buffer):
   fstopt ('MSGLVL',6)
   fstd2nc.stdout.streams = ('error',)
 
+  # Turn off Python warning / exception handling, use stdio for communication.
+  fstd2nc.stdout._python = False
+
   try:
     buf = buffer_type(infile, **args)
   except FSTDError:
@@ -97,12 +100,12 @@ def _fstdump (buffer_type=Buffer):
           values = var.array
         elif hasattr(var,'record_id'):
           import numpy as np
-          values = np.array([buf._fstluk(rec)['d'].T for rec in var.record_id.flatten()])
+          values = np.array([buf._read_record(rec) for rec in var.record_id.flatten()])
         elif hasattr(var,'chunks'):
           import numpy as np
           values = np.zeros(var.shape,var.dtype)
           for ind, rec in var.items():
-            values[ind] = buf._fstluk(rec)['d'].T
+            values[ind] = buf._read_record(rec)
         else:
            continue
         # Build string values?
@@ -170,6 +173,8 @@ def _fstd2nc_cmdline (buffer_type=Buffer):
     fstopt ('MSGLVL',6)
     fstd2nc.stdout.streams = ('error',)
     args['progress'] = False
+  else:
+    fstd2nc.stdout.streams = ('info','warn','error',)
   progress = args.get('progress',False)
   if use_pandas:
     fstd2nc.mixins._pandas_needed = True
@@ -181,6 +186,9 @@ def _fstd2nc_cmdline (buffer_type=Buffer):
     msglvl = {'DEBUG':0,'INFORM':2,'WARNIN':4,'ERRORS':6,'FATALE':8,'SYSTEM':10,'CATAST':10}[msglvl]
   fstopt ('MSGLVL',msglvl)
 
+  # Turn off Python warning / exception handling, use stdio for communication.
+  fstd2nc.stdout._python = False
+
   try:
     buf = buffer_type(infiles, **args)
   except FSTDError:
