@@ -241,7 +241,7 @@ class BufferBase (object):
 
   # Record parameters which should not be used as nc variable attributes.
   # (They're either internal to the file, or part of the data, not metadata).
-  _ignore_atts = ('file_id','key','name','address','length','dtype','selected')
+  _ignore_atts = ('file_id','name','address','length','dtype','selected')
 
   # Define any command-line arguments for reading FSTD files.
   @classmethod
@@ -287,14 +287,14 @@ class BufferBase (object):
   ###############################################
   # Basic flow for reading data
 
-  def __init__ (self, filename, _headers=None, progress=False, serial=False, **kwargs):
+  def __init__ (self, filename, progress=False, serial=False, **kwargs):
     """
     Read raw records from FSTD files, into the buffer.
     Multiple files can be read simultaneously.
 
     Parameters
     ----------
-    filename : str or list
+    filename : str or list or existing Buffer object
         The input files to convert.
     progress : bool, optional
         Display a progress bar during the conversion, if the "progress"
@@ -324,6 +324,13 @@ class BufferBase (object):
 
     self._serial = serial
 
+    # Detect if an existing Buffer object was provided.
+    if hasattr(filename,'_headers') and hasattr(filename,'_files'):
+      existing_buffer = filename
+      filename = []
+    else:
+      existing_buffer = None
+
     if isinstance(filename,(str,Path)):
       infiles = [filename]
     else:
@@ -410,7 +417,6 @@ class BufferBase (object):
       indices = np.concatenate(indices)
       headers = self._decode_headers(headers)
       # Encode the keys without the file index info.
-      headers['key'] = (indices % 256) | ((indices//256)<<9)
       headers['file_id'] = np.array(file_ids, dtype='int32')
       headers['selected'] = np.ones(len(file_ids),'bool')
 
@@ -428,21 +434,19 @@ class BufferBase (object):
         else:
           warn(_("Problem with input file '%s'")%infile)
 
+    # Use existing Buffer object data, if that was provided instead of files.
+    if existing_buffer is not None:
+      self._files = list(existing_buffer._files)
+      headers = {k:v.copy() for k,v in existing_buffer._headers.items()}
+
     nfiles = len(self._files)
-    if nfiles == 0:
+    if nfiles == 0 and len(infiles) > 0:
       error(_("no input files found!"))
     elif nfiles > 10:
       global _pandas_needed
       _pandas_needed = True
     info(_("Found %d %s"%(nfiles,self._format_plural)))
 
-    # Add extra headers? (hack for generating Buffer objects from external
-    # sources (not FSTD files)
-    if _headers is not None:
-      headers = dict(_headers)
-      # Keep link to first file for metadata purposes below?
-      headers['file_id'] = np.zeros(len(headers['nomvar']), dtype='int32')
-
     self._headers = headers
     self._nrecs = max(len(self._headers[key]) for key in self._headers.keys())
 
