@@ -56,27 +56,24 @@ class FSTD_Compat (BufferBase):
     fstd_compat = kwargs.pop('fstd_compat', False)
     if fstd_compat:
       self.to_netcdf = self._to_netcdf_compat
+      self._fstluk = self._fstluk_compat
     super(FSTD_Compat,self).__init__(*args,**kwargs)
 
   # Simulate 'fstluk' call (returning data + parameters).
   # This will also keep track of which FST records were used in the
   # conversion.  The header information for these records will be added to the
   # output file.
-  def _fstluk (self, rec_id, dtype=None, rank=None, dataArray=None):
-
-    # Extract record parameters.
-    prm = {k:v[rec_id] for k,v in self._headers.items()}
-
-    # Add data.
-    prm['d'] = self._read_record(rec_id).T
-    if dtype is not None:
-      prm['d'] = prm['d'].astype(dtype)
+  def _fstluk_compat (self, rec_id, dtype=None, rank=None, dataArray=None):
 
     # Keep track of which records were used.
     if not hasattr(self,'_used_rec_ids'):
       self._used_rec_ids = []
     self._used_rec_ids.append(rec_id)
 
+    prm = super(FSTD_Compat,self)._fstluk(rec_id)
+    if dtype is not None:
+      prm['d'] = prm['d'].astype(dtype)
+
     return prm
 
   def _to_netcdf_compat (self, filename, nc_format='NETCDF4', global_metadata=None, zlib=False, compression=4, progress=False, turbo=False):
