@@ -309,8 +309,9 @@ class ExternOutput (BufferBase):
     # Put all the header info into a dictionary.
     fields = ['nomvar', 'typvar', 'etiket', 'ni', 'nj', 'nk', 'dateo', 'ip1', 'ip2', 'ip3', 'deet', 'npas', 'datyp', 'nbits', 'grtyp', 'ig1', 'ig2', 'ig3', 'ig4', 'datev']
     table = dict()
-    # Create a mask to exclude deleted / overwritten records.
-    mask = self._headers['dltf'] == 0
+    # Create a mask to exclude deleted / overwritten / unselected records.
+    # Include all meta (coordinate) records in the output.
+    mask = self._headers['selected'] | self._headers['ismeta']
     for field in fields:
       col = self._headers[field][mask]
       # Convert byte arrays to strings, which is what fstpy expects.
@@ -325,7 +326,12 @@ class ExternOutput (BufferBase):
     table['shape'] = list(zip(table['ni'],table['nj']))
     filenames = dict((i,f) for i,f in enumerate(self._files))
     table['path'] = pd.Series(self._headers['file_id'][mask]).map(filenames)
-    table['key'] = (self._headers['key'][mask] << 10)
+    key = np.zeros(len(self._headers['name']),'int32')
+    for file_id in range(len(self._files)):
+      selection = self._headers['file_id'] == file_id
+      indices = np.arange(np.sum(selection), dtype='int32')
+      key[selection] = (indices % 256) | ((indices//256)<<9)
+    table['key'] = key[mask] << 10
     # Generate dask objects
     #TODO: use our own, in case we modified the data?
     # (doesn't normally happen, but you never know...)
@@ -352,11 +358,12 @@ def _fix_to_pygeode (fixed=[False]):
   fixed[0] = True
 
 class ExternInput (BufferBase):
+  def __init__ (self, *args, **kwargs):
+    if '_extern_table' in kwargs:
+      self._extern_table = kwargs.pop('_extern_table')
+    super(ExternInput,self).__init__(*args,**kwargs)
   @classmethod
   def from_fstpy (cls, table, **kwargs):
-    import tempfile
-    from os import path
-    import rpnpy.librmn.all as rmn
     import numpy as np
     if hasattr(table,'to_pandas'):
       table = table.to_pandas()
@@ -380,47 +387,46 @@ class ExternInput (BufferBase):
       # any subroutine is looking for this info.
       headers['address'] = np.empty(len(headers['nomvar']), dtype=object)
       headers['length'] = np.empty(len(headers['nomvar']), dtype=object)
-    # Generate temporary file with target grid info.
-    try: # Python 3
-      grid_tmpdir = tempfile.TemporaryDirectory()
-      gridfile = path.join(grid_tmpdir.name,"grid.fst")
-    except AttributeError: # Python 2 (no auto cleanup)
-      grid_tmpdir = tempfile.mkdtemp()
-      gridfile = path.join(grid_tmpdir,"grid.fst")
-    # Write all grid records to a temporary file, so they are accessible to
-    # the vgrid / librmn helper functions.
-    iun = rmn.fstopenall(gridfile, rmn.FST_RW)
-    for nomvar in (b'!!  ',b'>>  ',b'^^  ',b'^>  ',b'!!SF'):
-      for ind in np.where(headers['nomvar'] == nomvar)[0]:
-        rec = table.iloc[ind].to_dict()
-        rec['d'] = np.asarray(rec['d'])
-        rmn.fstecr(iun, rec)
-    rmn.fstcloseall(iun)
+    # Fake file id (just so netcdf mixin _quick_load function doesn't crash).
+    headers['file_id'] = np.zeros(len(headers['nomvar']), dtype='int32')
 
-    # Initialize the Buffer object with this info.
-    b = cls(gridfile, _headers=headers, **kwargs)
-    b._grid_tmpdir = grid_tmpdir  # Save tmpdir until cleanup.
+    # Encapsulate this info in a structure.
+    fake_buffer = cls.__new__(cls)
+    fake_buffer._files = [None]
+    fake_buffer._headers = headers
 
-    # Save the dataframe for reference.
+    # Initialize a Buffer object with this info.
+    # Also save the dataframe for reference.
     # Will need the dask objects for getting the data.
-    b._extern_table = table
+    b = cls(fake_buffer, _extern_table=table, **kwargs)
 
     return b
 
+  # Handle external data for read_record method.
+  # Use data from _extern_table.
+  def _read_record (self, rec_id):
+    import numpy as np
+    # Check if there is custom data enabled for this Buffer.
+    if hasattr(self, '_extern_table'):
+      # Extract the record info from the table.
+      rec = self._extern_table.iloc[rec_id].to_dict()
+      # Load the data (if delayed).
+      rec['d'] = np.asarray(rec['d'])
+      return rec['d'].T
+    # Otherwise, continue as usual.
+    return super(ExternInput,self)._read_record (rec_id)
+
   # Handle external data for _decode method.
   # In this case, the first argument is ignored (no file data was read).
   def _decode (self, maybe_data, rec_id):
     import numpy as np
     # Check if there is custom data enabled for this Buffer.
     if hasattr(self, '_extern_table'):
-      # Make sure we are looking for something in our list of records.
-      # (could be a key pointing into something else?)
-      if not isinstance(rec_id,dict):
-        # Extract the record info from the table.
-        rec = self._extern_table.iloc[rec_id].to_dict()
-        # Load the data (if delayed).
-        rec['d'] = np.asarray(rec['d'])
-        return rec['d'].T
+      # Extract the record info from the table.
+      rec = self._extern_table.iloc[rec_id].to_dict()
+      # Load the data (if delayed).
+      rec['d'] = np.asarray(rec['d'])
+      return rec['d'].T
     # Otherwise, continue as usual.
     return super(ExternInput,self)._decode (maybe_data, rec_id)
 
