@@ -21,6 +21,70 @@
 from fstd2nc.stdout import _, info, warn, error
 from fstd2nc.mixins import BufferBase
 
+############################################################
+# Mixin for questionable modifications to the headers table.
+#
+
+class GridHacks (BufferBase):
+  # Switch on hacks.
+  def _enable_hacks (self):
+    if not hasattr(self,'_hacks'):
+      self._hacks = dict()
+      self._read_record = self.__read_record
+      self._fstluk = self.__fstluk
+
+  # Hack in record data that doesn't actually exist in the source file.
+  def __read_record (self, rec_id):
+    if rec_id not in getattr(self,'_hacks',{}):
+      return super(GridHacks,self)._read_record(rec_id)
+    return self._hacks[rec_id]['d'].T
+
+  def __fstluk (self, rec_id):
+    if rec_id not in getattr(self,'_hacks',{}):
+      return super(GridHacks,self)._fstluk(rec_id)
+    return self._hacks[rec_id]
+
+  # Helper method - "write" a grid descriptor into the headers table.
+  # Based loosely on writeGrid from rpnpy.
+  def _writeGrid (self, gid):
+    import numpy as np
+    if gid['grtyp'] not in ('Z','#','Y','U'): return None
+    self._enable_hacks()
+    # Define some parameters for the grid.
+    prm = dict()
+    prm['typvar'] = 'X '
+    prm['ip1'] = gid['tag1']
+    prm['ip2'] = gid['tag2']
+    prm['ip3'] = gid.get('tag3',0)
+    prm['grtyp'] = gid['grref']
+    prm['ig1'] = gid['ig1ref']
+    prm['ig2'] = gid['ig2ref']
+    prm['ig3'] = gid['ig3ref']
+    prm['ig4'] = gid['ig4ref']
+    prm['datyp'] = gid.get('datyp',5)
+    prm['nbits'] = gid.get('nbits',32)
+    prm['etiket'] = gid.get('etiket','ETIKET').ljust(12)
+    prm['ni'] = 1; prm['nj'] = 1; prm['nk'] = 1
+    prm['ismeta'] = True
+    nrecs = len(self._headers['name'])
+    new_recs = []
+    if 'ax' in gid:
+      new_recs.append(dict(prm, nomvar='>>  ', ni=gid['ni'], d=gid['ax']))
+    if 'ay' in gid:
+      new_recs.append(dict(prm, nomvar='^^  ', nj=gid['nj'], d=gid['ay']))
+    if 'axy' in gid:
+      new_recs.append(dict(prm, nomvar='^>  ', ni=gid['ni'], nj=gid['nj'], d=gid['axy']))
+    for k,v in self._headers.items():
+      self._headers[k] = np.zeros_like(v, shape=nrecs + len(new_recs))
+      self._headers[k][:nrecs] = v[:]
+    for i in range(len(new_recs)):
+      for k,v in new_recs[i].items():
+        if k in self._headers:
+          if isinstance(v,str): v = v.encode()
+          self._headers[k][nrecs+i] = v
+      self._hacks[nrecs+i] = new_recs[i]
+    self._nrecs += len(new_recs)
+
 
 #################################################
 # Mixin for on-the-fly grid interpolation.
@@ -40,10 +104,9 @@ class Interp (BufferBase):
         Interpolate to the specified grid.
     """
     import rpnpy.librmn.all as rmn
-    import tempfile
-    from os import path
     import numpy as np
     interp = kwargs.pop('interp',None)
+    super(Interp,self).__init__(*args,**kwargs)
     # Extract interpolation grid.
     def number(x):
       try: return int(x)
@@ -57,72 +120,64 @@ class Interp (BufferBase):
       if not hasattr(rmn,'defGrid_'+grtyp):
         error(_("Unknown grid '%s'")%grtyp)
       self._interp_grid = getattr(rmn,'defGrid_'+grtyp)(*grid_args,**grid_kwargs)
-      # Generate temporary file with target grid info.
-      try: # Python 3
-        self._interp_tmpdir = tempfile.TemporaryDirectory()
-        gridfile = path.join(self._interp_tmpdir.name,"grid.fst")
-      except AttributeError: # Python 2 (no auto cleanup)
-        self._interp_tmpdir = tempfile.mkdtemp()
-        gridfile = path.join(self._interp_tmpdir,"grid.fst")
-      iun = rmn.fstopenall(gridfile, rmn.FST_RW)
-      rmn.writeGrid (iun, self._interp_grid)
-      rmn.fstcloseall (iun)
-      # Handle case where a single file was passed from the Python interface.
-      if isinstance(args[0],str):
-        args = list(args)
-        args[0] = [args[0]]
-        args = tuple(args)
-      # Add this grid file to the inputs.
-      args[0].append(gridfile)
-      super(Interp,self).__init__(*args,**kwargs)
-      # Update records to use this grid.
+      # Hack the new grid descriptors into the headers.
+      self._writeGrid (self._interp_grid)
+
+      # Store original and modified versions of the grid descriptors.
       ismeta = self._headers['ismeta']
+      self._original_grid = dict()
+      self._modified_grid = dict()
       for key in ('grtyp','ni','nj','ig1','ig2','ig3','ig4'):
-        self._headers[key][:] = np.where(ismeta, self._headers[key], self._interp_grid[key])
+        self._original_grid[key] = self._headers[key]
+        self._modified_grid[key] = np.array(self._headers[key])
+        self._modified_grid[key][:] = np.where(ismeta, self._headers[key], self._interp_grid[key])
+
       # Set up some options for ezsint.
       rmn.ezsetopt (rmn.EZ_OPT_EXTRAP_DEGREE, rmn.EZ_EXTRAP_VALUE)
       rmn.ezsetopt (rmn.EZ_OPT_EXTRAP_VALUE, self._fill_value)
-    else:
-      super(Interp,self).__init__(*args,**kwargs)
 
-  # Add fill value to the data.
-  # Modified from code in from mask mixin.
-  # Set this fill value for any interpolated data, since it may contain
-  # points outside the original boundary.
+
   def _makevars (self):
     from fstd2nc.mixins import _iter_type
+    import numpy as np
+
+    if not hasattr(self,'_interp_grid'):
+      return super(Interp,self)._makevars()
+
+    # Run _makevars chain with original grid descriptors to allow
+    # xycoords to give us source grid ids,
+    # switch out the grid descriptors in the table, then let xycoords
+    # construct the target grid axes for us.
     super(Interp,self)._makevars()
-    if hasattr(self,'_interp_grid'):
-      for var in self._varlist:
-        if isinstance(var,_iter_type):
-          var.atts['_FillValue'] = var.dtype.type(self._fill_value)
+    self._source_gids = np.array(self._gids)
+    self._headers.update(self._modified_grid)
+    super(Interp,self)._makevars()
+
+    # Add fill value to the data.
+    # Modified from code in from mask mixin.
+    # Set this fill value for any interpolated data, since it may contain
+    # points outside the original boundary.
+    for var in self._varlist:
+      if isinstance(var,_iter_type):
+        var.atts['_FillValue'] = var.dtype.type(self._fill_value)
 
   # Handle grid interpolations from raw binary array.
-  def _decode (self, data, unused, _grid_cache={}):
+  def _decode (self, data, rec_id, _grid_cache={}):
     import rpnpy.librmn.all as rmn
     import numpy as np
-    if not hasattr(self,'_interp_grid'):
-      return super(Interp,self)._decode (data, unused)
-    grid_params = ('ni','nj','grtyp','ig1','ig2','ig3','ig4')
-    prm = self._decode_headers(data[:72])
-    prm = dict((k,v[0]) for k,v in prm.items())
-    prm['d'] = super(Interp,self)._decode (data, unused).T
+    if self._headers['ismeta'][rec_id] or not hasattr(self,'_interp_grid'):
+      return super(Interp,self)._decode (data, rec_id)
+    # Retrieve an active librmn grid id associated with this grid.
+    # (must be supplied by xycoords mixin).
+    ingrid = int(self._source_gids[rec_id])
+    if ingrid < 0:
+      raise ValueError("Source data is not on a recognized grid.  Unable to interpolate.")
+    d = super(Interp,self)._decode (data, rec_id).T
     with self._lock:
-      cache_key = tuple(prm[k] for k in grid_params)
-      # Check if we've already defined the input grid in a previous call.
-      if cache_key in _grid_cache:
-        ingrid = _grid_cache[cache_key]
-      else:
-        ingrid = dict((k,(str(prm[k].decode()) if k=='grtyp' else int(prm[k]))) for k in grid_params)
-        if hasattr(self,'_meta_funit'):
-          ingrid['iunit'] = self._meta_funit
-        ingrid = rmn.ezqkdef (**ingrid)
-        _grid_cache[cache_key] = ingrid
-
       # Propogate any fill values to the interpolated grid.
-      in_mask = np.zeros(prm['d'].shape, order='F', dtype='float32')
-      in_mask[prm['d']==self._fill_value] = 1.0
-      d = rmn.ezsint (self._interp_grid, ingrid, prm['d'])
+      in_mask = np.zeros(d.shape, order='F', dtype='float32')
+      in_mask[d==self._fill_value] = 1.0
+      d = rmn.ezsint (self._interp_grid, ingrid, d)
       out_mask = rmn.ezsint (self._interp_grid, ingrid, in_mask)
       d[out_mask!=0] = self._fill_value
       # Return the data for the interpolated field.
@@ -150,11 +205,7 @@ class YinYang (BufferBase):
         Select second subgrid from a supergrid.
     """
     import rpnpy.librmn.all as rmn
-    import tempfile
-    from os import path
     import numpy as np
-    from fstd2nc.extra import structured_array
-    from collections import OrderedDict
     self._yin = kwargs.pop('yin',False)
     self._yang = kwargs.pop('yang',False)
 
@@ -165,49 +216,36 @@ class YinYang (BufferBase):
     if not self._yin and not self._yang:
       return
 
-    # Find all unique YY input grids.
-    mask = (self._headers['ismeta']==0) & (self._headers['grtyp']==b'U')
-    grids = OrderedDict((key,self._headers[key][mask]) for key in ('ig1','ig2','ig3','ig4'))
-    grids, rec_ids = np.unique(structured_array(grids), return_index=True)
-    rec_ids = np.where(mask)[0][rec_ids]
-    nomvars = self._headers['nomvar'][rec_ids]
-    source_grids = dict()
-    for nomvar, grid in zip(nomvars,grids):
-      ig1 = grid['ig1']
-      ig2 = grid['ig2']
-      ig3 = grid['ig3']
-      ig4 = grid['ig4']
-      for handle in rmn.fstinl(self._meta_funit, nomvar=nomvar.decode()):
-        prm = rmn.fstprm(handle)
-        if prm['grtyp'] == b'U' and prm['ig1'] == ig1 and prm['ig2'] == ig2 and prm['ig3'] == ig3 and prm['ig4'] == ig4:
-          break
-      source_grids[(ig1,ig2,ig3,ig4)] = rmn.readGrid (self._meta_funit, prm)
-
-    # Generate temporary file with target grid info.
-    try: # Python 3
-      self._yy_tmpdir = tempfile.TemporaryDirectory()
-      gridfile = path.join(self._yy_tmpdir.name,"grid.fst")
-    except AttributeError: # Python 2 (no auto cleanup)
-      self._yy_tmpdir = tempfile.mkdtemp()
-      gridfile = path.join(self._yy_tmpdir,"grid.fst")
-    iun = rmn.fstopenall(gridfile, rmn.FST_RW)
+    # Run _makevars early to generate grid ids with xycoords mixin.
+    # Silence warnings from makevars, which might not be relevant to the final
+    # state after we make our grid modifications.
+    import fstd2nc
+    streams = fstd2nc.stdout.streams
+    fstd2nc.stdout.streams = ()
+    self._makevars()
+    fstd2nc.stdout.streams = streams
+    gids = self._gids
 
+    # Get the Z grid for one half of the YY input.
     if self._yin: yy_ind = 0
     else: yy_ind = 1
-
-    # Write all target grids and set the headers appropriately.
-    for (ig1,ig2,ig3,ig4), source_grid in source_grids.items():
+    for gid in np.unique(gids):
+      if gid<0: continue
+      source_grid = rmn.decodeGrid(int(gid))
       dest_grid = source_grid['subgrid'][yy_ind]
-      rmn.writeGrid (iun, dest_grid)
+      self._writeGrid (dest_grid)
+      mask = (self._headers['ismeta']==0) & (self._headers['grtyp']==b'U')
+      # Need to get ig1/ig2/ig3/ig4 from a sample record, because apparently
+      # these values get mangles in librmn for U grids?
+      rec_id = np.where(gids==gid)[0][0]
+      ig1 = int(self._headers['ig1'][rec_id])
+      ig2 = int(self._headers['ig2'][rec_id])
+      ig3 = int(self._headers['ig3'][rec_id])
+      ig4 = int(self._headers['ig4'][rec_id])
       submask = mask & (self._headers['ig1'] == ig1) & (self._headers['ig2'] == ig2) & (self._headers['ig3'] == ig3) & (self._headers['ig4'] == ig4)
       for key in ('grtyp','ni','nj','ig1','ig2','ig3','ig4'):
         self._headers[key][submask] = dest_grid[key]
-    rmn.fstcloseall (iun)
 
-    # Hack in these extra coordinate records, and refresh metadata file unit.
-    rmn.fstcloseall (self._meta_funit)
-    self._meta_filenames.append(gridfile)
-    self._meta_funit = rmn.fstopenall(self._meta_filenames,rmn.FST_RO)
 
   # Handle grid interpolations from raw binary array.
   def _decode (self, data, unused):
@@ -240,8 +278,6 @@ class Crop (BufferBase):
     """
     import rpnpy.librmn.all as rmn
     import numpy as np
-    from fstd2nc.extra import structured_array
-    from collections import OrderedDict
     self._crop_to_smallest_grid = kwargs.pop('crop_to_smallest_grid',False)
 
     # Load the metadata from the file(s).
@@ -256,31 +292,24 @@ class Crop (BufferBase):
     self._headers['j0'] = np.zeros(self._nrecs,'uint16')
     self._headers['jN'] = np.array(self._headers['nj'],'uint16')
 
-    # Find all unique grids.
-    ###
-    # Skip supergrids.
-    mask = (self._headers['ismeta']==0) & (self._headers['grtyp'] != b'U')
-    grids = OrderedDict((key,self._headers[key][mask]) for key in ('ig1','ig2','ig3','ig4'))
-    grids, rec_ids = np.unique(structured_array(grids), return_index=True)
-    rec_ids = np.where(mask)[0][rec_ids]
-    source_grids = dict()
-    for rec_id in rec_ids:
-      prm = dict()
-      prm['grtyp'] = self._headers['grtyp'][rec_id].decode()
-      prm['ni'] = int(self._headers['ni'][rec_id])
-      prm['nj'] = int(self._headers['nj'][rec_id])
-      prm['ig1'] = int(self._headers['ig1'][rec_id])
-      prm['ig2'] = int(self._headers['ig2'][rec_id])
-      prm['ig3'] = int(self._headers['ig3'][rec_id])
-      prm['ig4'] = int(self._headers['ig4'][rec_id])
-      key = tuple(prm[k] for k in ('ig1','ig2','ig3','ig4'))
-      source_grids[key] = rmn.readGrid (self._meta_funit, prm)
+    # Run _makevars early to generate grid ids with xycoords mixin.
+    # Silence warnings from makevars, which might not be relevant to the final
+    # state after we make our grid modifications.
+    import fstd2nc
+    streams = fstd2nc.stdout.streams
+    fstd2nc.stdout.streams = ()
+    self._makevars()
+    fstd2nc.stdout.streams = streams
+    gids = self._gids
 
-    ###
-
-    # Group by ig1ref, etc.
+    # Find all available grids, group them by their projection parameters.
+    # I.e., lump together all grids that only differ in their x/y extent.
     matches = dict()
-    for grid in source_grids.values():
+    for gid in np.unique(gids):
+      if gid<0: continue
+      grid = rmn.decodeGrid(int(gid))
+      # Skip supergrids.
+      if grid['grtyp'] == 'U': continue
       key = (grid['ig1ref'],grid['ig2ref'],grid['ig3ref'],grid['ig4ref'])
       matches.setdefault(key,[]).append(grid)
 
@@ -293,28 +322,29 @@ class Crop (BufferBase):
 
     # Ok, now look at all the source grids, and figure out which ones are
     # candidates for cropping.
-    for (ig1,ig2,ig3,ig4), grid in source_grids.items():
-      key = (grid['ig1ref'],grid['ig2ref'],grid['ig3ref'],grid['ig4ref'])
+    for key in smallest.keys():
       smallest_grid = matches[key][smallest[key]]
-      # Nothing to do if already the smallest grid.
-      if grid['id'] == smallest_grid['id']: continue
-      # Make sure the coordinates are actually compatible.
-      ind = np.searchsorted(grid['ax'].flatten(), smallest_grid['ax'].flatten())
-      i0, iN = ind[0], ind[-1]+1
-      ind = np.searchsorted(grid['ay'].flatten(), smallest_grid['ay'].flatten())
-      j0, jN = ind[0], ind[-1]+1
-      if iN-i0 != smallest_grid['ni']: continue
-      if jN-j0 != smallest_grid['nj']: continue
-      if np.any(grid['ax'].flatten()[i0:iN] != smallest_grid['ax'].flatten()): continue
-      if np.any(grid['ay'].flatten()[j0:jN] != smallest_grid['ay'].flatten()): continue
-      # Able to crop, so update the headers to point to the cropped coordinates.
-      submask = mask & (self._headers['ig1'] == ig1) & (self._headers['ig2'] == ig2) & (self._headers['ig3'] == ig3) & (self._headers['ig4'] == ig4)
-      for key in ('grtyp','ni','nj','ig1','ig2','ig3','ig4'):
-        self._headers[key][submask] = smallest_grid[key]
-      self._headers['i0'][submask] = i0
-      self._headers['iN'][submask] = iN
-      self._headers['j0'][submask] = j0
-      self._headers['jN'][submask] = jN
+      for grid in matches[key]:
+        # Nothing to do if already the smallest grid.
+        if grid['id'] == smallest_grid['id']: continue
+        # Make sure the coordinates are actually compatible.
+        ind = np.searchsorted(grid['ax'].flatten(), smallest_grid['ax'].flatten())
+        i0, iN = ind[0], ind[-1]+1
+        ind = np.searchsorted(grid['ay'].flatten(), smallest_grid['ay'].flatten())
+        j0, jN = ind[0], ind[-1]+1
+        if iN-i0 != smallest_grid['ni']: continue
+        if jN-j0 != smallest_grid['nj']: continue
+        if np.any(grid['ax'].flatten()[i0:iN] != smallest_grid['ax'].flatten()): continue
+        if np.any(grid['ay'].flatten()[j0:jN] != smallest_grid['ay'].flatten()): continue
+        # Able to crop, so update the headers to point to the cropped coordinates.
+        submask = (self._headers['ig1'] == grid['tag1']) & (self._headers['ig2'] == grid['tag2']) & (self._headers['ig3'] == grid['tag3'])
+        for key in ('grtyp','ni','nj','ig1','ig2','ig3','ig4'):
+          self._headers[key][submask] = smallest_grid[key]
+        self._headers['i0'][submask] = i0
+        self._headers['iN'][submask] = iN
+        self._headers['j0'][submask] = j0
+        self._headers['jN'][submask] = jN
+
 
   # Handle cropping from raw binary array.
   def _decode (self, data, rec_id):
