@@ -67,13 +67,10 @@ class VarDict (BufferBase):
     vardicts = kwargs.pop('vardict',None) or []
     if kwargs.pop('opdict',False):
       f = environ['AFSISIO']+'/datafiles/constants/opdict/ops.variable_dictionary.xml'
-      vardicts.append(open(f,'r'))
+      vardicts.append(f)
 
     super(VarDict,self).__init__(*args, **kwargs)
 
-    # Open the files if only the filename is provided.
-    vardicts = [open(v,'r') if isinstance(v,str) else v for v in vardicts]
-
     metadata = OrderedDict()
     ip1_axis = OrderedDict()
     ip3_axis = OrderedDict()
@@ -85,6 +82,7 @@ class VarDict (BufferBase):
         error (_("Invalid dictionary file '%s'"%vardict.name))
 
       for metvar in metvars:
+        if metvar.attrib.get('usage','current') != 'current': continue
         nomvar = metvar.findtext('nomvar')
         if nomvar is None: continue
         var = metadata.setdefault(nomvar,OrderedDict())
