@@ -254,13 +254,11 @@ class RotLatLon(GridMap):
     self._lonatts['long_name'] = 'longitude'
     self._lonatts['standard_name'] = 'longitude'
     self._lonatts['units'] = 'degrees_east'
-    self._lonatts['axis'] = 'X'
     self._latarray = ll['lat'].transpose() # Switch from Fortran to C order.
     self._latatts = OrderedDict()
     self._latatts['long_name'] = 'latitude'
     self._latatts['standard_name'] = 'latitude'
     self._latatts['units'] = 'degrees_north'
-    self._latatts['axis'] = 'Y'
     self.lon = _var_type('lon',self._lonatts,self.gridaxes,self._lonarray)
     self.lat = _var_type('lat',self._latatts,self.gridaxes,self._latarray)
     # Get boundary info.
@@ -338,13 +336,11 @@ class PolarStereo(GridMap):
     self._lonatts['long_name'] = 'longitude'
     self._lonatts['standard_name'] = 'longitude'
     self._lonatts['units'] = 'degrees_east'
-    self._lonatts['axis'] = 'X'
     self._latarray = ll['lat'].transpose() # Switch from Fortran to C order.
     self._latatts = OrderedDict()
     self._latatts['long_name'] = 'latitude'
     self._latatts['standard_name'] = 'latitude'
     self._latatts['units'] = 'degrees_north'
-    self._latatts['axis'] = 'Y'
     xy = gdxyfll(self._grd['id'],ll['lat'],ll['lon'])
     # Scale grid coordinates back to actual coordinates in projection plane   
     self._ax = ( np.rint(xy['x'][:,0]) - 1) * self._res   # metres
@@ -456,20 +452,19 @@ class XYCoords (BufferBase):
   # them?
   def _find_coord (self, var, coordname):
     from fstd2nc.mixins.fstd import dtype_fst2numpy
-    from rpnpy.librmn.fstd98 import fstlir
     # Special case for series data - match any of the lat/lon grids.
     if var.atts['grtyp'] in ('+','Y'):
-      header = fstlir (self._meta_funit, nomvar=coordname, rank=3)
+      header = self._fstlir (nomvar=coordname)
       # Make sure this actually matches a grid of the correct shape.
       if header['ni'] != var.atts['ni'] or header['nj'] != var.atts['nj']:
         header = None
     else:
-      header = fstlir (self._meta_funit, nomvar=coordname, ip1=var.atts['ig1'],
-                             ip2=var.atts['ig2'], ip3=var.atts['ig3'], rank=3)
+      header = self._fstlir (nomvar=coordname, ip1=var.atts['ig1'],
+                             ip2=var.atts['ig2'], ip3=var.atts['ig3'])
     if header is not None:
       # Override output dtype
-      dtype = dtype_fst2numpy(header['datyp'],header['nbits'])
-      header['d'] = header['d'].view(dtype)
+      dtype = dtype_fst2numpy(int(header['datyp']),int(header['nbits']))
+      header['d'] = header['d'][:,:,None].view(dtype)
       return header
     raise KeyError("Unable to find matching '%s' for '%s'"%(coordname,var.name))
 
@@ -479,12 +474,17 @@ class XYCoords (BufferBase):
     from fstd2nc.mixins import _iter_type, _chunk_type, _var_type, _axis_type, _dim_type
     from collections import OrderedDict
     from rpnpy.librmn.interp import ezqkdef, EzscintError, ezget_nsubgrids
-    from rpnpy.librmn.all import readGrid, RMNError
+    from rpnpy.librmn.all import cxgaig, ezgdef_fmem, ezgdef_supergrid, ezqkdef, decodeGrid, RMNError
     import numpy as np
 
+    # Save a copy of librmn grid ids, which might be useful for other mixins.
+    self._gids = np.empty(self._nrecs,dtype=int)
+    self._gids[:] = -1
+
     # Scan through the data, and look for any use of horizontal coordinates.
     grids = OrderedDict()
     gridmaps = OrderedDict()
+    gids = OrderedDict()
     lats = OrderedDict()
     lons = OrderedDict()
     # Only output 1 copy of 1D coords (e.g. could have repetitions with
@@ -543,7 +543,24 @@ class XYCoords (BufferBase):
               atts['ig3'] = 1
               atts['ig4'] = 1
           try:
-            grd = readGrid(self._meta_funit, atts)
+            # Get grid object from librmn.
+            # Avoid readGrid, because it requires its own access to the
+            # files(s), which may not be readily available on disk.
+            if grtyp in ('Z','#'):
+              ref = self._find_coord(var,b'>>  ')
+              grd = atts.copy()
+              grd['grref'] = ref['grtyp']
+              grd['ig1'] = int(ref['ig1'])
+              grd['ig2'] = int(ref['ig2'])
+              grd['ig3'] = int(ref['ig3'])
+              grd['ig4'] = int(ref['ig4'])
+              grd['ax'] = ref['d'].squeeze()
+              grd['ay'] = self._find_coord(var,b'^^  ')['d'].squeeze()
+              grd = ezgdef_fmem(grd)
+            else:
+              grd = ezqkdef (ni, nj, grtyp, ig1, ig2, ig3, ig4)
+            gids[key] = grd
+            grd = decodeGrid(grd)
             gmap = GridMap.gen_gmap(grd,no_adjust_rlon=self._no_adjust_rlon)
             gmapvar = gmap.gen_gmapvar()
             gridmaps[key] = gmapvar
@@ -558,22 +575,46 @@ class XYCoords (BufferBase):
           latatts['long_name'] = 'latitude'
           latatts['standard_name'] = 'latitude'
           latatts['units'] = 'degrees_north'
-          latatts['axis'] = 'Y'
           lonatts = OrderedDict()
           lonatts['long_name'] = 'longitude'
           lonatts['standard_name'] = 'longitude'
           lonatts['units'] = 'degrees_east'
-          lonatts['axis'] = 'X'
 
           latarray = lonarray = None
           try:
             # First, handle non-ezqkdef grids.
             if grtyp in self._direct_grids:
-              latarray = self._find_coord(var,'^^')['d'].squeeze(axis=2)
-              lonarray = self._find_coord(var,'>>')['d'].squeeze(axis=2)
+              latarray = self._find_coord(var,b'^^  ')['d'].squeeze(axis=2)
+              lonarray = self._find_coord(var,b'>>  ')['d'].squeeze(axis=2)
             # Handle ezqkdef grids.
             else:
-              gdid = ezqkdef (ni, nj, grtyp, ig1, ig2, ig3, ig4, getattr(self,'_meta_funit',0))
+              ###
+              # U grids aren't yet supported by in-memory ezqkdef
+              # (requires ext
+              if grtyp == 'U':
+                ref = self._find_coord(var,b'^>  ')
+                data = ref['d'].flatten()
+                nsubgrids = int(data[2])
+                subgrids = []
+                subdata = data[5:]
+                for i in range(nsubgrids):
+                  sub_ni = int(subdata[0])
+                  sub_nj = int(subdata[1])
+                  # Loosely based on steps in Lire_enrUvercode1 of librmn.
+                  sub_ig1, sub_ig2, sub_ig3, sub_ig4 = cxgaig('E',*subdata[6:10])
+                  sub_ax = subdata[10:10+sub_ni]
+                  sub_ay = subdata[10+sub_ni:10+sub_ni+sub_nj]
+                  subgrids.append(ezgdef_fmem(sub_ni, sub_nj, 'Z', 'E', sub_ig1, sub_ig2, sub_ig3, sub_ig4, sub_ax, sub_ay))
+                  subdata = subdata[10+sub_ni+sub_nj:]
+                gdid = ezgdef_supergrid(ni, nj, 'U', 'F', 1, subgrids)
+              else:
+                #TODO: check if this case still gets triggered?
+                # GridMap grids (N,S,A,B,L,G,Z,E) are already handled.
+                # Direct grids (X,Y,T,+,O,M) are already handled.
+                # Supergrids (U) are already handled.
+                # What's left?
+                gdid = ezqkdef (ni, nj, grtyp, ig1, ig2, ig3, ig4, 0)
+              gids[key] = gdid
               ll = gdll(gdid)
               latarray = ll['lat']
               lonarray = ll['lon']
@@ -685,6 +726,12 @@ class XYCoords (BufferBase):
           warn(_("Unable to get lat/lon coordinates for '%s'")%var.name)
           continue
 
+        # If lat/lon are 1D axes, then add an 'axis' attribute.
+        if isinstance(lat,_axis_type):
+          lat.atts = dict(lat.atts, axis='Y')
+        if isinstance(lon,_axis_type):
+          lon.atts = dict(lon.atts, axis='X')
+
         # Sanity check on lat/lon - make sure we have something of the right size.
         if lat.array.shape == lat.shape and lon.array.shape == lon.shape:
           grids[key] = gridaxes
@@ -699,6 +746,12 @@ class XYCoords (BufferBase):
       gridaxes = grids[key]
       lat = lats[key]
       lon = lons[key]
+      # Store librmn grid ids for possible use in other parts of code.
+      if key in gids and isinstance(var,_iter_type):
+        ind = var.record_id.flatten()
+        ind = ind[ind>=0]
+        if len(ind) > 0:
+          self._gids[ind] = gids[key]
 
       # Update the var's horizontal coordinates.
       newaxes = []
